/*
 *  Plugins/Plugins.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using System.Reflection;
using System.Xml;
using System.IO;
using Mono.Unix;
using Incollector.Engine;
using Gtk;
using ICSharpCode.SharpZipLib.Zip;


namespace Incollector.Plugins {

	public enum InstallationResult {
		Successful    = 0,
		PluginUpdated = 1,
		AlreadyExists = 2,
		OlderVersion  = 3,
		Dependences   = 4,
		NotKnown      = 5
	}


	/*
	 * Plugin class
	 */
	
	public static class Plugins {
	
		public struct PluginStruct {
			public string  FileName;
			public Version Ver;
			
			public PluginStruct (string file, Version version)
			{
				this.FileName = file;
				this.Ver      = version;
			}
		}
		
		public static ArrayList PluginsList = new ArrayList ();
		public static ArrayList AllPlugins  = new ArrayList ();
		
		
		/* Loads all plugins from user and global plugins directory */ 
		public static void LoadAllPlugins ()
		{
			GetPluginsList (".");
			
			if (Directory.Exists (Engine.FilesPaths.GlobalPluginsDirectory))
				GetPluginsList (Engine.FilesPaths.GlobalPluginsDirectory);
			
			if (Directory.Exists (Engine.FilesPaths.UserPluginsDirectory))
				GetPluginsList (Engine.FilesPaths.UserPluginsDirectory);
			
			
			for (int i = 0; i < AllPlugins.Count; i++) {
				PluginStruct plug = (PluginStruct) AllPlugins [i];
				LoadPlugin (plug.FileName);
			}
		}
		
		
		/* Returns plugins list */
		private static void GetPluginsList (string directory)
		{
			DirectoryInfo [] pgns = new DirectoryInfo (directory).GetDirectories ();
			if (pgns.Length == 0)
				return;
			
			foreach (DirectoryInfo pgn in pgns) {
				string pgnpath = Path.Combine (pgn.FullName, pgn.Name + ".dll");
				string infpath = Path.Combine (pgn.FullName, pgn.Name + ".xml");
				
				Version version = new PluginInfo (infpath).version;
				
				for (int i = 0; i < AllPlugins.Count; i++) {
					PluginStruct plugin = (PluginStruct) AllPlugins [i];
					
					if (Path.GetFileName (plugin.FileName) == Path.GetFileName (pgnpath)) {
						if (version > plugin.Ver) {
							plugin.FileName = pgnpath;
							plugin.Ver      = version;
							AllPlugins [i]  = plugin;
						}
						
						goto next;
					}
				}
				
				AllPlugins.Add (new PluginStruct (pgnpath, version));
				
				next: ;
			}
		}
		
		
		/* Loads plugin */
		public static void LoadPlugin (string PluginFile)
		{
			try {
				Assembly asm = Assembly.LoadFrom (PluginFile);
				foreach (Type type in asm.GetTypes ()) {
					if (type.IsSubclassOf (typeof (Plugin))) {
						// loading plugin
						Plugin plugin;
						
						try {
							plugin = (Plugin) Activator.CreateInstance (type);
							plugin.Init ();
						} catch (System.Exception exc) {
							GUI.PluginErrorDialog dlg = new GUI.PluginErrorDialog (
								Path.GetFileName (PluginFile), PluginFile, exc.StackTrace);
							dlg.dialog.Run ();
							dlg.dialog.Destroy ();
							
							continue;
						}
						
						plugin.Path = PluginFile;
						PluginsList.Add (plugin);
					}
				}
			} catch {
			}
		}
		
		
		public static InstallationResult InstallPlugin (string FileName, out ArrayList Dependences)
		{
			Dependences = new ArrayList ();
			
			// unpacking
			System.Random rand = new System.Random ();
			string UnpackDirectory = Path.Combine (FilesPaths.UserPluginsDirectory, Path.GetFileName (FileName) + ".tmp." + rand.Next (1000, 100000).ToString ());
			
			FastZip zipfile = new FastZip ();
			zipfile.ExtractZip (FileName, UnpackDirectory, "");
			
			DirectoryInfo [] dir = new DirectoryInfo (UnpackDirectory).GetDirectories ();
			if (dir.Length != 1)
				return InstallationResult.NotKnown;
			
			string pgndir  = FilesPaths.UserPluginsDirectory + FilesPaths.Separator + dir [0].Name;
			string pgnfile = dir [0].Name + ".dll";
			string infpath = dir [0].FullName  + FilesPaths.Separator + dir [0].Name + ".xml";
			
			
			// checking version
			PluginInfo info = new PluginInfo (infpath);
			Version version = info.version;
			
			// checking dependences
			ArrayList deps = info.CheckDependences ();
			if (deps.Count > 0) {
				Dependences = deps;
				Directory.Delete (UnpackDirectory, true);
				return InstallationResult.Dependences;
			} else
				Dependences = new ArrayList ();
			
			
			foreach (PluginStruct plugin in AllPlugins) {
				if (string.Compare (Path.GetFileName (plugin.FileName), pgnfile) == 0) {
					if (version < plugin.Ver) { // newer version exists
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.OlderVersion;
					} else if (version == plugin.Ver) { // the same version already exists
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.AlreadyExists;
					} else { // older version exists -> updating !
						if (plugin.FileName.StartsWith (FilesPaths.UserPluginsDirectory) == true) { // if not user plugins dir
							try {
								Directory.Delete (pgndir, true);
							} catch {}
						}
						
						try {
							System.IO.Directory.Move (dir [0].FullName, pgndir);
						} catch {}
						
						Directory.Delete (UnpackDirectory, true);
						return InstallationResult.PluginUpdated;
					}
				}
			}
			
			
			try {
				System.IO.Directory.Move (dir [0].FullName, pgndir);
			} catch {}
			
			Directory.Delete (UnpackDirectory, true);
			return InstallationResult.Successful;
		}
		
		
		public static bool UninstallPlugin (string PluginPath)
		{
//			string pgnfile = PluginPath.Remove (PluginPath.Length - 
//				Path.GetExtension (PluginPath).Length, Path.GetExtension (PluginPath).Length);
//			
//			if (pgnfile.StartsWith (FilesPaths.UserPluginsDirectory) == false)
//				return false;
//			
//			
//			try {
//				Directory.Delete (pgnfile, true);
//			} catch {}
//			
//			try {
//				File.Delete (pgnfile + ".dll");
//			} catch {}
//			
//			try {
//				File.Delete (pgnfile + ".xml");
//			} catch {}
			
			return true;
		}
	}
}
